*---------------------------------------------------------------------* 00010000
*                                                                     * 00020000
* Name:        REXXWAIT                                               * 00030000
*                                                                     * 00040000
* Function:    REXX Function to wait for hhmmssth                     * 00050000
*                                                                     * 00060000
* Syntax:      X=REXXWAIT(hhmmssth)                                   * 00070000
*                                                                     * 00080000
*              Where:                                                 * 00090000
*                                                                     * 00100000
*              hhmmssth is:                                           * 00110000
*              hh = hours 01 to nn                                    * 00120000
*              mm = minutes 00 to 59                                  * 00130000
*              ss = seconds 00 to 59                                  * 00140000
*              t  = tenths of a second 0-9                            * 00150000
*              h  = hundredths of a second 0-9                        * 00160000
*                                                                     * 00170000
*              Default is to wait for 15 seconds                      * 00180000
*---------------------------------------------------------------------* 00190002
* Sample REXX Usage:                                                  * 00200002
*                                                                     * 00210000
*  /* rexx */                                                         * 00220003
*  arg val                                                            * 00230003
*  call time 'r'                                                      * 00240003
*  x=rexxwait(val)                                                    * 00250003
*  etime = time('e')                                                  * 00260003
*  say "Elapsed time:" etime "rc:" x                                  * 00270003
*                                                                     * 00280002
*---------------------------------------------------------------------* 00290000
* Link Edit information:     NON-REENTRANT                            * 00300000
*---------------------------------------------------------------------* 00310000
* Return Codes:                                                       * 00320000
*                                                                     * 00330000
*         0    processing has completed successfully                  * 00340000
*         8    invalid parm                                           * 00350000
*        24    IRXEXCOM bad return code                               * 00360000
*                                                                     * 00370000
*---------------------------------------------------------------------* 00380000
* Author:      Lionel B. Dyck                                         * 00390000
*              Kaiser Permanente                                      * 00400000
*              Walnut Creek, CA 94 98                                 * 00410000
*              (925) 926-5332                                         * 00420000
*              Internet: lionel.b.dyck@kp.org                         * 00430000
*---------------------------------------------------------------------* 00440000
* HISTORY:                                                            * 00450000
*                                                                     * 00460000
*        07/22/99  -  Created                                         * 00470000
*                                                                     * 00480000
*---------------------------------------------------------------------* 00490000
REXXWAIT CSECT                                                          00500000
REXXWAIT AMODE 31                                                       00510000
*----------------------------------------------------------*            00520000
*        Housekeeping occurs here                          *            00530000
*----------------------------------------------------------*            00540000
         STM   R14,R12,12(R13)     SAVE REGISTERS                       00550000
         LR    R12,R15             LOAD BASE REG                        00560000
         USING REXXWAIT,R12        INFORM ASSEMBLER                     00570000
         B     STARTIT                                                  00580000
         DC    CL8'REXXWAIT'                                            00590000
         DC    C'-'                                                     00600000
         DC    CL8'&SYSDATE'       COMPILE DATE                         00610000
         DC    C'-'                                                     00620000
         DC    CL8'&SYSTIME'       COMPILE TIME                         00630000
STARTIT  DS    0H                                                       00640000
         SPACE 2                                                        00650000
         LA    R0,72+1000          get savearea plus workarea           00660000
         GETMAIN R,LV=(0)                                               00670000
         MVI   0(R1),X'00'         MOVE X'00' TO FIRST BYTE             00680000
         LR    R2,R1               SAVE POINTER IN EVEN REG             00690000
         LA    R4,1(R1)            SET RECEIVING POINTER                00700000
         LR    R5,R0               SET RECEIVING LENGTH                 00710000
         BCTR  R5,R0               DECREMENT LENGTH                     00720000
         LA    R5,0(R5)            CLEAR HIGH ORDER BYTE                00730000
         LA    R3,1                SET SENDING LENGTH                   00740000
         MVCL  R4,R2               INSTRUCTION PADS WITH X'00'          00750000
         ST    R13,4(R1)           SAVE BACK CHAIN                      00760000
         ST    R1,8(R13)           SET FORWARD CHAIN                    00770000
         LR    R2,R1               SAVE NEW SAVEAREA ADDRESS            00780000
         L     R15,16(R13)         RESTORE REG 15                       00790000
         ST    R0,16(R13)          SAVE SAVEAREA LENGTH                 00800000
         LM    R0,R1,20(R13)       RESTORE REGS USED IN GETMAIN         00810000
         LR    R13,R2              SET SAVEAREA POINTER                 00820000
         USING WORK,R13            USING ON WORKAREA VIA R13            00830000
         EJECT                                                          00840000
*---------------------------------------------------------------------* 00850000
*        PROCESS THE INPUT PARAMETER LIST                             * 00860000
*---------------------------------------------------------------------* 00870000
         LR    R5,R0               -> ENVIRONMENT BLOCK                 00880000
         ST    R5,ENVBADDR         Save Environment Block Address       00890000
         USING ENVBLOCK,R5                                              00900000
         L     R5,ENVBLOCK_IRXEXTE -> EXTERNAL VECTOR TABLE             00910000
         DROP  R5                                                       00920000
         USING IRXEXTE,R5                                               00930000
         L     R5,IRXEXCOM         LOAD IRXEXCOM EPA                    00940000
         ST    R5,EXCOM            SAVE IRXEXCOM EPA                    00950000
         DROP  R5                                                       00960000
         LR    R4,R1               -> EFPL                              00970000
         USING EFPL,R4             ESTABLISH ADDRESSABILITY TO EFPL     00980000
         L     R5,EFPLEVAL         -> EVAL BLOCK ADDRESS POINTER        00990000
         L     R5,0(0,R5)          -> EVAL BLOCK                        01000000
         ST    R5,EVALADDR         SAVE IT'S ADDRESS                    01010000
         L     R4,EFPLARG          -> ARGUMENT LIST                     01020000
         DROP  R4                                                       01030000
*                                                                       01040000
         LTR   R4,R4               ANY ARGUMENT LIST ?                  01050000
         BZ    SETDEF              NO                                   01060000
*                                                                       01070000
         USING ARGTABLE_ENTRY,R4   ESTABLISH ADDRESSABILITY TO ARG      01080000
         CLC   =8X'FF',ARGTABLE_ARGSTRING_PTR  ANY REAL ARGUMENTS       01090000
         BE    SETDEF              NO - SO EXIT                         01100000
         L     R2,ARGTABLE_ARGSTRING_PTR    -> ARGUMENT                 01110000
         L     R3,ARGTABLE_ARGSTRING_LENGTH LOAD LENGTH                 01120000
         C     R3,=F'08'           Test length                          01130001
         BNE   EXIT8               Invalid parm                         01140001
         BCTR  R3,R0               LESS 1 FOR MVC                       01150000
         EX    R3,MOVETIME         MOVE THE TIME                        01160000
         B     START                                                    01170000
SETDEF   MVC   WAITTIME(8),=CL8'00001500'                               01180000
         EJECT                                                          01190000
*---------------------------------------------------------------------* 01200000
*        INITIALIZE ALL WORK AREAS FROM OUR CONSTANTS                 * 01210000
*---------------------------------------------------------------------* 01220000
START    DS    0H                                                       01230000
*                                                                       01240000
         STIMER WAIT,DINTVL=WAITTIME                                    01250000
         SPACE 2                                                        01260000
         B     EXIT0                                                    01270000
         EJECT                                                          01280000
*---------------------------------------------------------------------* 01290000
*        ALL OF THE EXIT ROUTINES FOLLOW                              * 01300000
*---------------------------------------------------------------------* 01310000
SETRC    DS    0H                                                       01320000
         L     R11,EVALADDR                                             01330000
         USING EVALBLOCK,R11                                            01340000
         MVC   EVALBLOCK_EVLEN(4),=A(2)                                 01350000
         MVC   EVALBLOCK_EVDATA(2),RC                                   01360000
         DROP  R11                                                      01370000
         SPACE 2                                                        01380000
         LR    R1,R13              GET SAVEAREA ADDRESS                 01390000
         L     R13,4(R13)          GET BACK CHAIN POINTER               01400000
         L     R0,16(R13)          GET SAVEAREA LENGTH                  01410000
         ST    R15,16(R13)         SAVE REGISTER 15 (RETCODE)           01420000
         FREEMAIN R,LV=(0),A=(1)   FREE SAVEAREA                        01430000
         LM    R14,R12,12(R13)     RESTORE CALLERS REGS                 01440000
         LA    R15,0                                        $1/95$      01450000
         BR    R14                                                      01460000
*---------------------------------------------------------------------* 01470000
EXIT0    DS    0H                                                       01480000
         MVC   RC,=C'00'           SET RC TO ZERO                       01490000
         B     SETRC                                                    01500000
EXIT8    DS    0H                                                       01510001
         MVC   RC,=C'08'           SET RC TO 8                          01520001
         B     SETRC                                                    01530001
EXIT24   DS    0H                                                       01540000
         MVC   RC,=C'24'           SET RC                               01550000
         B     SETRC                                                    01560000
         EJECT                                                          01570000
*---------------------------------------------------------------------* 01580000
*        EXECUTED INSTRUCTIONS                                        * 01590000
*---------------------------------------------------------------------* 01600000
MOVETIME MVC   WAITTIME(0),0(R2)       ** EXECUTED                      01610000
         SPACE 2                                                        01620000
*---------------------------------------------------------------------* 01630000
*        LITERAL POOL                                                 * 01640000
*---------------------------------------------------------------------* 01650000
         LTORG                                                          01660000
         space 2                                                        01670000
*---------------------------------------------------------------------* 01680000
*        WORK AREA DSECTS                                             * 01690000
*---------------------------------------------------------------------* 01700000
WORK     DSECT                                                          01710000
SAVE     DS    18F                                                      01720000
EVALADDR DS    F                                                        01730000
EXCOM    DS    A                   ADDRESS OF IRXEXCOM LOAD MODULE      01740000
ENVBADDR DS    A                   Environment Block Address            01750000
WAITTIME DS    CL8                 Time value                           01760000
RC       DS    XL2                 RETURN CODE                          01770000
         EJECT                                                          01780000
         IRXEFPL DSECT=YES                                              01790000
         EJECT                                                          01800000
         IRXARGTB                                                       01810000
         EJECT                                                          01820000
         IRXSHVB                                                        01830000
         EJECT                                                          01840000
         IRXEVALB                                                       01850000
         EJECT                                                          01860000
         IRXEXTE                                                        01870000
         EJECT                                                          01880000
         IRXENVB                                                        01890000
         EJECT                                                          01900000
R0       EQU   0                                                        01910000
R1       EQU   1                                                        01920000
R2       EQU   2                                                        01930000
R3       EQU   3                                                        01940000
R4       EQU   4                                                        01950000
R5       EQU   5                                                        01960000
R6       EQU   6                                                        01970000
R7       EQU   7                                                        01980000
R8       EQU   8                                                        01990000
R9       EQU   9                                                        02000000
R10      EQU   10                                                       02010000
R11      EQU   11                                                       02020000
R12      EQU   12                                                       02030000
R13      EQU   13                                                       02040000
R14      EQU   14                                                       02050000
R15      EQU   15                                                       02060000
         END   ,                                                        02070000
